@@ -4,10 +4,10 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
 #include <fcntl.h>
 #include <sys/types.h>
 
+#include "helpers.h"
 #include "liburing.h"
 
 #define IOVECS_LEN 2
@@ -34,7 +34,7 @@ int main(int argc, char *argv[])
 	}
 
 	for (i = 0; i < IOVECS_LEN; ++i) {
-		iovecs[i].iov_base = malloc(64);
+		iovecs[i].iov_base = t_malloc(64);
 		iovecs[i].iov_len = 64;
 	};
 
