@@ -22,7 +22,7 @@
 
 int main(int argc, char *argv[])
 {
-	int p_fd[2];
+	int p_fd[2], ret;
 	int32_t recv_s0;
 	int32_t val = 1;
 	struct sockaddr_in addr;
@@ -30,37 +30,54 @@ int main(int argc, char *argv[])
 	if (argc > 1)
 		return 0;
 
+	srand(getpid());
+
 	recv_s0 = socket(AF_INET, SOCK_STREAM | SOCK_CLOEXEC, IPPROTO_TCP);
 
-	assert(setsockopt(recv_s0, SOL_SOCKET, SO_REUSEPORT, &val, sizeof(val)) != -1);
-	assert(setsockopt(recv_s0, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val)) != -1);
+	ret = setsockopt(recv_s0, SOL_SOCKET, SO_REUSEPORT, &val, sizeof(val));
+	assert(ret != -1);
+	ret = setsockopt(recv_s0, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val));
+	assert(ret != -1);
 
 	addr.sin_family = AF_INET;
-	addr.sin_port = 0x1235;
 	addr.sin_addr.s_addr = 0x0100007fU;
 
-	assert(bind(recv_s0, (struct sockaddr*)&addr, sizeof(addr)) != -1);
-	assert(listen(recv_s0, 128) != -1);
+	do {
+		addr.sin_port = (rand() % 61440) + 4096;
+		ret = bind(recv_s0, (struct sockaddr*)&addr, sizeof(addr));
+		if (!ret)
+			break;
+		if (errno != EADDRINUSE) {
+			perror("bind");
+			exit(1);
+		}
+	} while (1);
+	ret = listen(recv_s0, 128);
+	assert(ret != -1);
 
 
 	p_fd[1] = socket(AF_INET, SOCK_STREAM | SOCK_CLOEXEC, IPPROTO_TCP);
 
 	val = 1;
-	assert(setsockopt(p_fd[1], IPPROTO_TCP, TCP_NODELAY, &val, sizeof(val)) != -1);
+	ret = setsockopt(p_fd[1], IPPROTO_TCP, TCP_NODELAY, &val, sizeof(val));
+	assert(ret != -1);
 
 	int32_t flags = fcntl(p_fd[1], F_GETFL, 0);
 	assert(flags != -1);
 
 	flags |= O_NONBLOCK;
-	assert(fcntl(p_fd[1], F_SETFL, flags) != -1);
+	ret = fcntl(p_fd[1], F_SETFL, flags);
+	assert(ret != -1);
 
-	assert(connect(p_fd[1], (struct sockaddr*)&addr, sizeof(addr)) == -1);
+	ret = connect(p_fd[1], (struct sockaddr*)&addr, sizeof(addr));
+	assert(ret == -1);
 
 	flags = fcntl(p_fd[1], F_GETFL, 0);
 	assert(flags != -1);
 
 	flags &= ~O_NONBLOCK;
-	assert(fcntl(p_fd[1], F_SETFL, flags) != -1);
+	ret = fcntl(p_fd[1], F_SETFL, flags);
+	assert(ret != -1);
 
 	p_fd[0] = accept(recv_s0, NULL, NULL);
 	assert(p_fd[0] != -1);
@@ -69,7 +86,8 @@ int main(int argc, char *argv[])
 		int32_t code;
 		socklen_t code_len = sizeof(code);
 
-		assert(getsockopt(p_fd[1], SOL_SOCKET, SO_ERROR, &code, &code_len) != -1);
+		ret = getsockopt(p_fd[1], SOL_SOCKET, SO_ERROR, &code, &code_len);
+		assert(ret != -1);
 
 		if (!code)
 			break;
@@ -77,7 +95,8 @@ int main(int argc, char *argv[])
 
 	struct io_uring m_io_uring;
 
-	assert(io_uring_queue_init(32, &m_io_uring, 0) >= 0);
+	ret = io_uring_queue_init(32, &m_io_uring, 0);
+	assert(ret >= 0);
 
 	char recv_buff[128];
 	char send_buff[128];
@@ -106,7 +125,8 @@ int main(int argc, char *argv[])
 		io_uring_prep_writev(sqe, p_fd[1], iov, 1, 0);
 	}
 
-	assert(io_uring_submit_and_wait(&m_io_uring, 2) != -1);
+	ret = io_uring_submit_and_wait(&m_io_uring, 2);
+	assert(ret != -1);
 
 	struct io_uring_cqe* cqe;
 	uint32_t head;
