@@ -3,7 +3,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/syscall.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <unistd.h>
@@ -31,10 +30,13 @@ static void loop(void)
 	exit(ret);
 }
 
-int main(void)
+int main(int argc, char *argv[])
 {
 	int i, ret, status;
 
+	if (argc > 1)
+		return 0;
+
 	for (i = 0; i < 12; i++) {
 		if (!fork()) {
 			loop();
